<?php

/**
 * PC Shop common helper module
 */

if (!defined('ERP_PATH')) {
  $erp_path = substr(drupal_get_path('module', 'erp_job'), 0, -8);
  $erp_images = $erp_path .'/images';

  define('ERP_PATH', $erp_path);
  define('ERP_IMAGES', $erp_images);

  define('ERP_WEIGHT', 0);
  define('ERP_PAGE', 50);
  define('ERP_AUTOCOMPLETE', 25);
}

theme_add_style(ERP_PATH .'/erp.css');

function shrink($text, $shrink = 0) {
  if (strlen($text) > $shrink + 2) {
    return substr($text, 0, $shrink) . '..';
  }
  else {
    return ($text ? $text : '');
  }
}


/**
 * Validate a job/timesheet/whatever's owner
 *
 * @param unknown_type $node
 * @return unknown
 */
function erp_owner_validate($name) {
  // Try and retrieve the uid for the job owner based on the name
  $owner = user_load(array('name' => $name));

  return ($owner->uid ? $owner->uid : 0);
}

function erp_oid_validate($oid) {
  // Try and retrieve the owner name from the uid
  $owner = user_load(array('uid' => $oid));

  return ($owner->name ? $owner->name : '');
}

/**
 * Debugging function to list variables
 *
 * @param unknown_type $var
 * @return unknown
 */
function erp_var_list($var, $html = true) {
  $string = '';

  if (is_string($var)) {
    $string = $var;
  }
  elseif (is_array($var)) {
    foreach($var as $key => $value) {
      $string .= ($html ? '<li>' : '') . $key . ' : ';
      if (is_array($value) or is_object($value)) {
        $string .= ($html ? '<ul>' : '') .
          erp_var_list($value, $html) .
          ($html ? '</ul></li>' : "\n");
      }
      else {
        $string .= $value . ($html ? '</li>' : "\n");
      }
    }
  }
  elseif (is_object($var)) {
    foreach(get_object_vars($var) as $key => $value) {
      $string .= ($html ? '<li>' : '') . $key . ($html ? ' : <ul>' : '');
      $string .= erp_var_list($value, $html) . ($html ? '</ul></li>' : '');
    }
  }
  else {
    $string .= "\n" . ($html ? '<BR>' : '');
  }
  return $string;
}

function erp_find_path_query($url) {
  global $base_url, $base_path;

  $source = parse_url($url);
  $path = $source['path'];
  $new_base = substr(substr($base_path, 1), 0, -1);
  $path = str_replace($base_path, '', $path);
  $query = $source['query'];
  return array($path, $query);
}

/**
 * Function used to both display and set the next invoice number,
 * po number, etc. see invoice_settings, purchase_order_settings
 * Defaults to collapsed as its not common
 *
 * TODO: I dont think this code will work with postgresql
 *
 * @param unknown_type $type
 * @param unknown_type $desc
 * @return unknown
 */
function erp_set_next_id_form($form, $type, $desc) {
  $op = $_POST['op'];

  if ($op == t('Set number')) {
    db_query("REPLACE INTO {sequences} (name, id) VALUES ('%s', %d)", $type, $_POST['edit'][$type .'_number']);
  }

  $result = db_query("SELECT id FROM {sequences} WHERE name = '%s'", $type);
  if (db_num_rows($result)) {
    $invoice_id = array_pop(db_fetch_array($result));
  }

  $form[$type .'_settings']['set_next'] =
    array('#type' => 'fieldset',
          '#title' => t('Next ') . $desc . t(' number'),
          '#collapsible' => TRUE,
          '#collapsed' => TRUE,
          '#description' => t('You can set the number for the next created ') . $desc . t(' here.'));

  $form[$type .'_settings']['set_next'][$type .'_number'] =
    array('#type' => 'textfield',
          '#size' => 10,
          '#maxlength' => 10,
          '#title' => t('Next ') . $desc . t(' number'),
          '#default_value' => $invoice_id);

  $form[$type .'_settings']['set_next'][$type .'_button'] =
    array('#type' => 'button',
          '#value' => t('Set number'));

  return $form;
}

function erp_add_newline() {
  $edit = $_POST['edit'];

  if ($_POST['addline'] == t('Add line')) {
    $newline = $edit['newline'];

    if ($newline['item_desc']) {
      list($supplier_code, $stock_code, $desc, $price) = erp_item_desc_breakup($newline['item_desc']);
      $item_nid = item_locate($supplier_code, $stock_code);
      $item_node = node_load($item_nid);

      $edit['entries']['item'][] = $item_nid;
      $edit['entries']['item_desc'][] = erp_item_desc_build($item_node);
      $edit['entries']['qty'][] = $newline['qty'];
      $edit['entries']['extra'][] = $newline['extra'];
      $edit['entries']['price'][] = $newline['price'];

      $edit['newline']['item'] = '';
      $edit['newline']['item_desc'] = '';
      $edit['newline']['qty'] = '';
      $edit['newline']['extra'] = '';
      $edit['newline']['price'] = '';
    }
  }
  return $edit;
}

/**
 * Helper function to display invoice entries.
 *
 * @param unknown_type $form
 * @return unknown
 */
function theme_entries($form) {
  $header = array(t('Qty'), t('Stock Code'), t('Date'), t('Extra Text'), t('Price'));
  $rows = array();

  foreach (element_children($form['qty']) as $key) {
    $rows[] = array(form_render($form['line'][$key]) . form_render($form['qty'][$key]),
                    form_render($form['item_desc'][$key]) . form_render($form['item'][$key]),
                    form_render($form['completed_date'][$key]),
                    form_render($form['extra'][$key]),
                    form_render($form['price'][$key]));
  }
  $rows[] = array(array(), array(), array(), array(), form_render($form['total']));

  $output .= '<div class="entries">'. theme('table', $header, $rows) . '</div>';

  $output .= form_render($form);

  return $output;
}

/**
 * Helper function to display invoice entries.
 *
 * @param unknown_type $form
 * @return unknown
 */
function theme_entries_no_date($form) {
  $header = array(t('Qty'), t('Stock Code'), t('Extra Text'), t('Price'));
  $rows = array();

  foreach (element_children($form['qty']) as $key) {
    $rows[] = array(form_render($form['line'][$key]) . form_render($form['qty'][$key]),
                    form_render($form['item_desc'][$key]) . form_render($form['item'][$key]),
                    form_render($form['extra'][$key]),
                    form_render($form['price'][$key]));
  }
  $rows[] = array(array(), array(), array(), form_render($form['total']));

  $output .= '<div class="entries">'. theme('table', $header, $rows) . '</div>';

  $output .= form_render($form);

  return $output;
}

/**
 * Theme newline entries
 */
function theme_newline($form) {
  $header = array();
  $rows = array();

  $rows[] = array(form_render($form['qty']),
                  form_render($form['item_desc']),
                  form_render($form['completed_date']),
                  form_render($form['extra']),
                  form_render($form['price']));

  $output .= '<div class="newline">'. theme('table', $header, $rows) . '</div>';

  $output .= form_render($form);

  return $output;
}
