<?php

/**
 * Adds an authentication token to the database.
 *
 * This function takes data from a variety of sources, producing an ID which a
 * client can use to identify with the server.  The nature of the data
 * collected means that the client <b>must</b> be on the same IP address if
 * it wants to authenticate.
 *
 * \todo Work out what to do about NAT.
 *
 * \return A string which must be used to authenticate, or NULL on error
 *
 * \see asset_auth_onetime
 */
function erp_asset_auth_add() {
	// Use the current session id as the authentication
	$hash = $_COOKIE['PHPSESSID'];

	// Add the current IP to allow you to audit multiple machines at once.
	$hash = $hash . $HTTP_SERVER_VARS['REMOTE_ADDR'];

  //Add a unique ID so that if the client-side chokes we don't
  //need to clear out the database manually.
  $hash = uniqid($hash, true);

  //Finally, MD5 the hash
  $hash = md5($hash);

	$sql = "INSERT INTO {erp_asset_auth} (hash)	VALUES ('%s')";

	$result = db_query($sql, $hash);
	$error = db_error($result);
	if ($error) 	{
		return array("$error");
		return NULL;
	}

	return $hash;
}

/**
 * Authenticates with the database, invalidating the key afterwards.
 *
 * This function uses the key from asset_auth_add() to decide whether or not
 * the client should be allowed to do anything.  If the client does not hold
 * the correct key or connect from the correct IP address, it will not hash to
 * the correct value, denying access.
 *
 * \param key		The key returned by asset_auth_add()
 *
 * \return			A true or false value if the client is permitted or denied
 *							respectively, or NULL on an error.
 *
 * \see asset_auth_add
 */
function erp_asset_auth_onetime($hash) {
	$sql = "SELECT registered FROM {erp_asset_auth} WHERE hash = '%s'";

	$result = db_query($sql, $hash);
	if(db_error($result)) {
		return NULL;	$ip = $_SERVER['REMOTE_ADDR'];
	}

	$rowcount = db_num_rows($result);
	if ($rowcount == 1) { // One match.  All is well.
		$sql = "DELETE FROM {erp_asset_auth} WHERE hash = '%s'";

		db_query($sql, $hash);
		return TRUE;
	}
	else if ($rowcount == 0) { // Nothing matches.  Incorrect key
		return FALSE;
	}
	else {	// Some md5/PHPSESSION+ip type conflict. Barf
	  return NULL;
	}
}

/**
 * Creates an ID for the computer.
 *
 * This function creates a unique ID suitable for identification purposes.
 * The ID returned however is not suitable for human consumption.  This ID
 * should be modified to include some sort of check digit when being used
 * by humans.
 *
 * \return	The numeric id, or NULL on error
 */
function erp_asset_create_cid() {
	$random = rand(0, 32767);
	$sql = "INSERT INTO {erp_asset_cid}	(retrieval_key) VALUES (%d)";

	$result = db_query($sql, $random);
	if (db_error($result)) {
		return NULL;
	}

	$sql = <<<SQL

		SELECT
			cid
		FROM
			{erp_asset_cid}
		WHERE
			retrieval_key = %d

SQL;

		$result = db_query($sql, $random);
		if (db_error($result)) {
			return NULL;
		}

		if (db_num_rows($result) != 1) {
			return NULL;
		}

		$id = db_result($result);

		$sql = <<<SQL

				UPDATE
					{erp_asset_cid}
				SET
					retrieval_key = 0
				WHERE
					cid = %d

SQL;
		db_query($sql, $id);

		return $id;
}

/**
 * Adds a check digit onto the ID, making it verifiable for typos.
 *
 * \param id	The ID to make verifiable
 * \return 		The ID with a check digit appended
 */
function erp_asset_cid_humanise($id) {
	$checkstring = (string)($id << 2);
	return ((string)$id) . $checkstring[0];
}

/**
 * Checks the modified ID to make sure that it does not contain typographical
 * errors.
 *
 * This function works similarly to asset_cid_humanise, effectively doing the
 * transformation again and checking to see if it ended the same way.
 *
 * \param id	The ID to check
 *
 * \return		A boolean indicating whether the ID is valid
 */
function erp_asset_cid_verify($id) {
	$cid = (string)$id;
	$base = (integer)substr($cid, 0, strlen($cid) - 1);
	$checkdigit = (string)($base << 2);
	$checkdigit = $checkdigit[0];
	if ($id[strlen($id) - 1] == $checkdigit) {
		return TRUE;
	}
	return FALSE;
}

/**
 * Gets the unique identifier for the computer's owning customer/group.
 *
 * This function looks for the identifier for the ownership info passed, and
 * either returns an already-existing identifier or creates a new one.
 *
 * \param	customer	The id of the customer
 * \param group     The id of the group
 * \param noadd    	Don't add it into the database if it's not there, just
 *                    return NULL
 *
 * \return An integer which can be used to identify the ownership
 *
 * \see asset_find_model_id
 */
function erp_asset_find_ownership_id($customer, $noadd = FALSE) {
	$sql = <<<SEARCHSQL

			SELECT
				modelid
			FROM
				{erp_asset_ownership_category}
			WHERE
				{erp_asset_ownership_category}.customerid = %d

SEARCHSQL;

	$result = db_query($sql, $customer);

	/* Something went wrong, lets just gracefully make it someone else's job */
	if (db_error($result)) {
		return NULL;
	}

	$row_count = db_num_rows($result);
	if ($row_count == 1) { //This has already been inserted
		$domain_id = db_result($result);
		return $domain_id;
	}
	else if ($row_count == 0) { //Never seen this data before, lets add it
		if ($noadd) {//We're not allowed to add it, so complain to whoever called us
			return NULL;
		}

		$sql = <<<ADDSQL

			INSERT INTO
				{erp_asset_ownership_category}
				(customerid)
			VALUES
				(%d)

ADDSQL;

		db_query($sql, $customer);
		return erp_asset_find_ownership_id($customer, TRUE); //Infinite loop == bad
	}
	else { //We should never get here unless the table is corrupt
		return NULL;
	}

}

/**
 * Gets the unique identifier for the computer's domain.
 *
 * This function looks for the identifier for the domain passed, and either
 * returns an already-existing identifier or creates a new one.  Abstracting
 * the domain names out facilitates changing the domain for all the records at
 * once, so <b>don't</b> store a raw domain name from anywhere outside this
 * function.
 *
 * \param	domain	The name of the domain to get an identifier for.
 * \param noadd 	Don't add it into the database if it's not there, just return
 * 								NULL
 *
 * \return An integer which can be used to identify a domain name.
 *
 * \see asset_find_model_id
 */
function erp_asset_find_domain_id($domain, $noadd = FALSE) {
	$sql = <<<SEARCHSQL

			SELECT
				modelid
			FROM
				{erp_asset_domain_category}
			WHERE
				{erp_asset_domain_category}.name = '%s'

SEARCHSQL;

	$result = db_query($sql, $domain);

	/* Something went wrong, lets just gracefully make it someone else's job */
	if (db_error($result)) {
		return NULL;
	}

	$row_count = db_num_rows($result);
	if ($row_count == 1) { //This domain has already been inserted
		$domain_id = db_result($result);
		return $domain_id;
	}
	else if ($row_count == 0) { //Never seen this domain before, lets add it
		if ($noadd) { //We're not allowed to add it, so complain to whoever called us
			return NULL;
		}

		$sql = <<<ADDSQL

			INSERT INTO
				{erp_asset_domain_category}
				(name)
			VALUES
				('%s')

ADDSQL;

		db_query($sql, $domain);
		return erp_asset_find_domain_id($domain, TRUE); //Infinite loop == bad
	}
	else { //We should never get here unless the table is corrupt
		return NULL;
	}

}

/**
 * Gets the unique identifier for the computer's SCSI adaptor
 *
 * This function looks for the identifier for the adaptor passed, and either
 * returns an already-existing identifier or creates a new one.
 *
 * \param	name		The name of the adaptor to get an identifier for.
 * \param noadd 	Don't add it into the database if it's not there, just return
 * 								NULL
 *
 * \return An integer which can be used to identify a domain name.
 *
 * \see asset_find_model_id
 */
function erp_asset_find_scsi_id($name, $noadd = FALSE) {
	$sql = <<<SEARCHSQL

			SELECT
				modelid
			FROM
				{erp_asset_scsi_category}
			WHERE
				{erp_asset_scsi_category}.name = '%s'

SEARCHSQL;

	$result = db_query($sql, $name);

	/* Something went wrong, lets just gracefully make it someone else's job */
	if(db_error($result))
	{
		return NULL;
	}

	$row_count = db_num_rows($result);
	if($row_count == 1) //This has already been inserted
	{
		$scsi_id = db_result($result);
		return $scsi_id;
	}
	elseif($row_count == 0) //Never seen this before, lets add it
	{
		if($noadd) //We're not allowed to add it, so complain to whoever called us
		{
			return NULL;
		}

		$sql = <<<ADDSQL

			INSERT INTO
				{erp_asset_scsi_category}
				(name)
			VALUES
				('%s')

ADDSQL;

		$result = db_query($sql, $name);
		if(db_error($result))
		{
			return NULL;
		}
		return erp_asset_find_scsi_id($name, TRUE); //Infinite loop == bad
	}
	else //We should never get here unless the table is corrupt
	{
		return NULL;
	}

}

/**
 * Gets the unique identifier for the computer's VGA adaptor
 *
 * This function looks for the identifier for the adaptor passed, and either
 * returns an already-existing identifier or creates a new one.
 *
 * \param	name		The name of the adaptor to get an identifier for.
 * \param noadd 	Don't add it into the database if it's not there, just return
 * 								NULL
 *
 * \return An integer which can be used to identify an adaptor
 *
 * \see asset_find_model_id
 */
function erp_asset_find_vga_id($name, $noadd = FALSE)
{
	$sql = <<<SEARCHSQL

			SELECT
				modelid
			FROM
				{erp_asset_vga_category}
			WHERE
				{erp_asset_vga_category}.name = '%s'

SEARCHSQL;

	$result = db_query($sql, $name);

	/* Something went wrong, lets just gracefully make it someone else's job */
	if(db_error($result))
	{
		return NULL;
	}

	$row_count = db_num_rows($result);
	if($row_count == 1) //This has already been inserted
	{
		$scsi_id = db_result($result);
		return $scsi_id;
	}
	elseif($row_count == 0) //Never seen this before, lets add it
	{
		if($noadd) //We're not allowed to add it, so complain to whoever called us
		{
			return NULL;
		}

		$sql = <<<ADDSQL

			INSERT INTO
				{erp_asset_vga_category}
				(name)
			VALUES
				('%s')

ADDSQL;

		$result = db_query($sql, $name);
		if(db_error($result))
		{
			return NULL;
		}
		return erp_asset_find_vga_id($name, TRUE); //Infinite loop == bad
	}
	else //We should never get here unless the table is corrupt
	{
		return NULL;
	}

}

/**
 * Gets the unique identifier for the computer's sound adaptor
 *
 * This function looks for the identifier for the adaptor passed, and either
 * returns an already-existing identifier or creates a new one.
 *
 * \param	name		The name of the adaptor to get an identifier for.
 * \param noadd 	Don't add it into the database if it's not there, just return
 * 								NULL
 *
 * \return An integer which can be used to identify an adaptor
 *
 * \see asset_find_model_id
 */
function erp_asset_find_sound_id($name, $noadd = FALSE)
{
	$sql = <<<SEARCHSQL

			SELECT
				modelid
			FROM
				{erp_asset_sound_category}
			WHERE
				{erp_asset_sound_category}.name = '%s'

SEARCHSQL;

	$result = db_query($sql, $name);

	/* Something went wrong, lets just gracefully make it someone else's job */
	if(db_error($result))
	{
		return NULL;
	}

	$row_count = db_num_rows($result);
	if($row_count == 1) //This has already been inserted
	{
		$sound_id = db_result($result);
		return $sound_id;
	}
	elseif($row_count == 0) //Never seen this before, lets add it
	{
		if($noadd) //We're not allowed to add it, so complain to whoever called us
		{
			return NULL;
		}

		$sql = <<<ADDSQL

			INSERT INTO
				{erp_asset_sound_category}
				(name)
			VALUES
				('%s')

ADDSQL;

		$result = db_query($sql, $name);
		if(db_error($result))
		{
			return NULL;
		}
		return erp_asset_find_sound_id($name, TRUE); //Infinite loop == bad
	}
	else //We should never get here unless the table is corrupt
	{
		return NULL;
	}

}

/**
 * Gets the unique identifier for the computer's CDROM drives
 *
 * This function looks for the identifier for the drive passed, and either
 * returns an already-existing identifier or creates a new one.
 *
 * \param	name		The name of the drive to get an identifier for.
 * \param noadd 	Don't add it into the database if it's not there, just return
 * 								NULL
 *
 * \return An integer which can be used to identify a drive name.
 *
 * \see asset_find_model_id
 */
function erp_asset_find_cdrom_id($name, $noadd = FALSE)
{
	$sql = <<<SEARCHSQL

			SELECT
				modelid
			FROM
				{erp_asset_cdrom_category}
			WHERE
				{erp_asset_cdrom_category}.name = '%s'

SEARCHSQL;

	$result = db_query($sql, $name);

	/* Something went wrong, lets just gracefully make it someone else's job */
	if(db_error($result))
	{
		return NULL;
	}

	$row_count = db_num_rows($result);
	if($row_count == 1) //This has already been inserted
	{
		$scsi_id = db_result($result);
		return $scsi_id;
	}
	elseif($row_count == 0) //Never seen this before, lets add it
	{
		if($noadd) //We're not allowed to add it, so complain to whoever called us
		{
			return NULL;
		}

		$sql = <<<ADDSQL

			INSERT INTO
				{erp_asset_cdrom_category}
				(name)
			VALUES
				('%s')

ADDSQL;

		db_query($sql, $name);
		return erp_asset_find_cdrom_id($name, TRUE); //Infinite loop == bad
	}
	else //We should never get here unless the table is corrupt
	{
		return NULL;
	}

}

/**
 * Gets the model id based on the vendor, model, and version.
 *
 * This function abstracts the vendor, model, and version information into a
 * single number for each model.  Doing this allows us to change this
 * information all at once (eg. so that when automatically getting a vendor's
 * contact information for a certain product occurs, it doesn't fail because
 * of a changed company name) and to make collection all of the assets based
 * on a particular model cheaper in terms of the database query.
 *
 * \param vendor	A string containing the vendor of the model
 *
 * \param model		A string containing the model of the product
 *
 * \param version	A string containing the product version/revision
 *
 * \param noadd		A boolean value specifying whether or not an entry can be
 *								added if the chosen model doesn't exist.
 *
 * \return				An integer containing a handle to the model information.
 *
 * \see asset_find_domain_id
 *
 */
function erp_asset_find_model_id($vendor, $model, $version, $noadd = FALSE)
{
	$sql = <<<FINDSQL

			SELECT
				modelid
			FROM
				{erp_asset_model_category}
			WHERE
				vendor='%s'
					AND
				model='%s'
					AND
				version='%s'

FINDSQL;

	$result = db_query($sql, $vendor, $model, $version);

	if(db_error($result)) //We couldn't search for anything.  Not Our Problem.
	{
		return NULL;
	}

	$row_count = db_num_rows($result);
	if($row_count == 1) //Model is already defined
	{
		$model_id = db_result($result);
		return $model_id;
	}
	elseif($row_count == 0) //No entry for this model, let's add it
	{
		$sql = <<<ADDSQL

			INSERT INTO
				{erp_asset_model_category}
				(vendor, model, version)
			VALUES
				( '%s', '%s', '%s' )

ADDSQL;
		if(!$noadd)
		{
			$result = db_query($sql, $vendor, $model, $version);
			if(db_error($result))
			{
				return NULL;
			}
		}
		else
		{
			return NULL;
		}

		return erp_asset_find_model_id($vendor, $model, $version, TRUE);
	}
	else //More than one row?  This should never, ever happen
	{
	}
}

/**
 * Gets the unique identifier for the computer's CPU.
 *
 * This function looks for the identifier for the CPU passed, and either
 * returns an already-existing identifier or creates a new one.  Abstracting
 * the CPU names out facilitates changing the domain for all the records at
 * once, so <b>don't</b> store a raw CPU from anywhere outside this function.
 *
 * \param name		The name of the CPU to get an identifier for.
 * \param speed		The speed of the CPU
 * \param noadd 	Don't add it into the database if it's not there, just return
 * 								NULL
 *
 * \return An integer which can be used to identify a CPU.
 *
 * \see asset_find_model_id
 */
function erp_asset_find_processor_id($name, $speed, $noadd = FALSE)
{
	$sql = <<<SEARCHSQL

			SELECT
				modelid
			FROM
				{erp_asset_processor_category}
			WHERE
				{erp_asset_processor_category}.name		=	'%s'
				AND
				{erp_asset_processor_category}.speed	=	%d

SEARCHSQL;

	$result = db_query($sql, $name, $speed);

	/* Something went wrong, lets just gracefully make it someone else's job */
	if(db_error($result))
	{
		return NULL;
	}

	$row_count = db_num_rows($result);
	if($row_count == 1) //This has already been inserted
	{
		$domain_id = db_result($result);
		return $domain_id;
	}
	elseif($row_count == 0) //Never seen this before, lets add it
	{
		if($noadd) //We're not allowed to add it, so complain to whoever called us
		{
			return NULL;
		}

		$sql = <<<ADDSQL

			INSERT INTO
				{erp_asset_processor_category}
				(name, speed)
			VALUES
				('%s', %d)

ADDSQL;

		db_query($sql, $name, $speed);
		return erp_asset_find_processor_id($name, $speed, TRUE); //Infinite loop == bad
	}
	else //We should never get here unless the table is corrupt
	{
		return NULL;
	}
}

/**
 * Gets the unique identifier for the computer's Hard Drive.
 *
 * This function looks for the identifier for the HD passed, and either
 * returns an already-existing identifier or creates a new one.
 *
 * \param name		The name of the domain to get an identifier for.
 * \param size		The size of the drive
 * \param noadd 	Don't add it into the database if it's not there, just return
 * 								NULL
 *
 * \return An integer which can be used to identify a hard drive
 *
 * \see asset_find_model_id
 */
function erp_asset_find_hd_id($name, $size, $noadd = FALSE)
{
	$sql = <<<SEARCHSQL

			SELECT
				modelid
			FROM
				{erp_asset_harddisk_category}
			WHERE
				{erp_asset_harddisk_category}.name		=	'%s'
				AND
				{erp_asset_harddisk_category}.size	=	%d

SEARCHSQL;

	$result = db_query($sql, $name, $size);

	/* Something went wrong, lets just gracefully make it someone else's job */
	if(db_error($result))
	{
		return NULL;
	}

	$row_count = db_num_rows($result);
	if($row_count == 1) //This has already been inserted
	{
		$hd_id = db_result($result);
		return $hd_id;
	}
	elseif($row_count == 0) //Never seen this before, lets add it
	{
		if($noadd) //We're not allowed to add it, so complain to whoever called us
		{
			return NULL;
		}

		$sql = <<<ADDSQL

			INSERT INTO
				{erp_asset_harddisk_category}
				(name, size)
			VALUES
				('%s', %d)

ADDSQL;

		db_query($sql, $name, $size);
		return erp_asset_find_hd_id($name, $size, TRUE); //Infinite loop == bad
	}
	else //We should never get here unless the table is corrupt
	{
		return NULL;
	}
}


/**
 * Gets the unique identifier for the computer's OS.
 *
 * This function looks for the identifier for the OS passed, and either
 * returns an already-existing identifier or creates a new one.  Abstracting
 * the OS names out facilitates changing the data for all the records at
 * once, so <b>don't</b> store raw data from anywhere outside this function.
 *
 * \param	name		The name of the OS to get an identifier for.
 * \param noadd 	Don't add it into the database if it's not there, just return
 * 								NULL
 *
 * \return An integer which can be used to identify a domain name.
 *
 * \see asset_find_model_id
 */
function erp_asset_find_os_id($name, $noadd = FALSE)
{
	$sql = <<<SEARCHSQL

			SELECT
				modelid
			FROM
				{erp_asset_os_category}
			WHERE
				{erp_asset_os_category}.name		=	'%s'

SEARCHSQL;

	$result = db_query($sql, $name);

	/* Something went wrong, lets just gracefully make it someone else's job */
	if(db_error($result))
	{
		return NULL;
	}

	$row_count = db_num_rows($result);
	if($row_count == 1) //This has already been inserted
	{
		$os_id = db_result($result);
		return $os_id;
	}
	elseif($row_count == 0) //Never seen this before, lets add it
	{
		if($noadd) //We're not allowed to add it, so complain to whoever called us
		{
			return NULL;
		}

		$sql = <<<ADDSQL

			INSERT INTO
				{erp_asset_os_category}
				(name)
			VALUES
				('%s')

ADDSQL;

		db_query($sql, $name);
		return erp_asset_find_os_id($name, TRUE); //Infinite loop == bad
	}
	else //We should never get here unless the table is corrupt
	{
		return NULL;
	}
}

/**
 * Gets the unique identifier for the computer's software.
 *
 * This function looks for the identifier for the data passed, and either
 * returns an already-existing identifier or creates a new one.
 *
 * \param	name		The name of the software to get an identifier for.
 * \param version	The version of the software to get an identifier for.
 * \param noadd 	Don't add it into the database if it's not there, just
 * 								return
 *
 * \return				An integer which can be used to identify the software.
 *
 * \see asset_find_model_id
 */
function erp_asset_find_sw_id($name, $version, $noadd = FALSE)
{
	$sql = <<<SEARCHSQL

			SELECT
				modelid
			FROM
				{erp_asset_software_category}
			WHERE
				{erp_asset_software_category}.name					=	'%s'
					AND
				{erp_asset_software_category}.sw_version		=	'%s'

SEARCHSQL;

	$result = db_query($sql, $name, $version);

	/* Something went wrong, lets just gracefully make it someone else's job */
	if(db_error($result))
	{
		return NULL;
	}

	$row_count = db_num_rows($result);
	if($row_count == 1) //This has already been inserted
	{
		$sw_id = db_result($result);
		return $sw_id;
	}
	elseif($row_count == 0) //Never seen this before, lets add it
	{
		if($noadd) //We're not allowed to add it, so complain to whoever called us
		{
			return NULL;
		}

		$sql = <<<ADDSQL

			INSERT INTO
				{erp_asset_software_category}
				(name, sw_version)
			VALUES
				('%s', '%s')

ADDSQL;

		db_query($sql, $name, $version);
		return erp_asset_find_sw_id($name, $version, TRUE); //Infinite loop == bad
	}
	else //We should never get here unless the table is corrupt
	{
		return NULL;
	}
}

/**
 * Gets the unique identifier for the NIC
 *
 * This function looks for the identifier for the NIC passed, and either
 * returns an already-existing identifier or creates a new one.
 *
 * \param	name		The name of the domain to get an identifier for.
 * \param noadd 	Don't add it into the database if it's not there, just return
 * 								NULL
 *
 * \return An integer which can be used to identify an NIC.
 *
 * \see asset_find_model_id
 */
function erp_asset_find_nic_id($name, $noadd = FALSE)
{
	$debug = variable_get('asset_debug', FALSE);
	$sql = <<<SEARCHSQL

			SELECT
				modelid
			FROM
				{erp_asset_nic_category}
			WHERE
				{erp_asset_nic_category}.name = '%s'

SEARCHSQL;

	$result = db_query($sql, $name);

	/* Something went wrong, lets just gracefully make it someone else's job */
	if(db_error($result))
	{
		return NULL;
	}

	$row_count = db_num_rows($result);
	if($row_count == 1) //This has already been inserted
	{
		$retval = db_result($result);
		return $retval;
	}
	elseif($row_count == 0) //Never seen this before, lets add it
	{
		if($noadd) //We're not allowed to add it, so complain to whoever called us
		{
			return NULL;
		}

		$sql = <<<ADDSQL

			INSERT INTO
				{erp_asset_nic_category}
				(name)
			VALUES
				('%s')

ADDSQL;

		db_query($sql, $name);
		return erp_asset_find_nic_id($name, TRUE); //Infinite loop == bad
	}
	else //We should never get here unless the table is corrupt
	{
		return NULL;
	}

}


/**
 * Inserts data regarding the processor.
 *
 * This function inserts information into the database which is involved with
 * the processor.  This doesn't do anything else, however it will insert the
 * processor model category as well, since this is required.
 *
 * \param computer	The computer's unique ID
 * \param name			The processor's name
 * \param speed			The speed of the processor
 */
function erp_asset_insert_processor($computer, $name, $speed)
{
	$processor_id = asset_find_processor_id($name, $speed);
	iF(NULL == $processor_id)
	{
		return;
	}

	$sql = <<<INSERTSQL

		INSERT INTO
			{erp_asset_processor}
			(computerid, modelid)
		VALUES
			(%d, %d)

INSERTSQL;

	db_query($sql, $computer, $processor_id);
}

/**
 * Inserts data regarding the SCSI adaptors
 *
 * This function inserts information into the database which is involved with
 * the SCSI adaptors.  This doesn't do anything else, however it will insert
 * the model category as well, since this is required.
 *
 * \param computer	The computer's unique ID
 * \param name			The adaptor's name
 */
function erp_asset_insert_scsi($computer, $name)
{
	$scsi_id = erp_asset_find_scsi_id($name);
	iF(NULL == $scsi_id)
	{
		return;
	}

	$sql = <<<INSERTSQL

		INSERT INTO
			{erp_asset_scsi}
			(computerid, modelid)
		VALUES
			(%d, %d)

INSERTSQL;

	db_query($sql, $computer, $scsi_id);
}

/**
 * Inserts data regarding the VGA adaptors
 *
 * This function inserts information into the database which is involved with
 * the VGA adaptors.  This doesn't do anything else, however it will insert
 * the model category as well, since this is required.
 *
 * \param computer	The computer's unique ID
 * \param name			The adaptor's name
 */
function erp_asset_insert_vga($computer, $name)
{
	$vga_id = erp_asset_find_vga_id($name);
	iF(NULL == $vga_id)
	{
		return;
	}

	$sql = <<<INSERTSQL

		INSERT INTO
			{erp_asset_vga_controller}
			(computerid, modelid)
		VALUES
			(%d, %d)

INSERTSQL;

	db_query($sql, $computer, $vga_id);
}

/**
 * Inserts data regarding the sound adaptors
 *
 * This function inserts information into the database which is involved with
 * the sound adaptors.  This doesn't do anything else, however it will insert
 * the model category as well, since this is required.
 *
 * \param computer	The computer's unique ID
 * \param name			The adaptor's name
 */
function erp_asset_insert_sound($computer, $name)
{
	$sound_id = erp_asset_find_sound_id($name);
	iF(NULL == $sound_id)
	{
		return;
	}

	$sql = <<<INSERTSQL

		INSERT INTO
			{erp_asset_sound_controller}
			(computerid, modelid)
		VALUES
			(%d, %d)

INSERTSQL;

	db_query($sql, $computer, $sound_id);
}

/**
 * Inserts data regarding the CDROM drives.
 *
 * This function inserts information into the database which is involved with
 * the CDROM drives.  This doesn't do anything else, however it will insert the
 * model category as well, since this is required.
 *
 * \param computer	The computer's unique ID
 * \param name			The drive's name
 */
function erp_asset_insert_cdrom($computer, $name)
{
	$cdrom_id = erp_asset_find_cdrom_id($name);
	iF(NULL == $cdrom_id)
	{
		return;
	}

	$sql = <<<INSERTSQL

		INSERT INTO
			{erp_asset_cdrom}
			(computerid, modelid)
		VALUES
			(%d, %d)

INSERTSQL;

	db_query($sql, $computer, $cdrom_id);
}

/**
 * Inserts data regarding the hard drives.
 *
 * This function inserts information into the database which is involved with
 * the hard drives.  This doesn't do anything else, however it will insert the
 * processor model category as well, since this is required.
 *
 * \param computer	The computer's unique ID
 * \param name			The hard drive's name
 * \param size			The amount of storage on the drive
 */
function erp_asset_insert_hd($computer, $name, $size)
{
	$debug = variable_get('asset_debug', FALSE);
	$hd_id = erp_asset_find_hd_id($name, $size);
	iF(NULL == $hd_id)
	{
		return;
	}


	$sql = <<<INSERTSQL

		INSERT INTO
			{erp_asset_harddisk}
			(computerid, modelid)
		VALUES
			(%d, %d)

INSERTSQL;

	db_query($sql, $computer, $hd_id);
}

/**
 * Inserts data about the RAM.
 *
 * This function inserts data which involves the RAM.  This only holds the size
 * at this time, but other platforms may supply more information, so this API
 * is subject to change if the client is ported.
 *
 * \param computer	The ID of the computer containing the RAM
 * \param amount		The amount of RAM that the computer holds (in MiB)
 */
function erp_asset_insert_ram($computer, $amount)
{
	$sql = <<<RAMSQL

		INSERT INTO
			{erp_asset_ram}
			(computerid, modelid)
		VALUES
			(%d, %d)

RAMSQL;
	db_query($sql, $computer, $amount);
}

/**
 * Inserts data about the ownership of the computer.
 *
 * This function inserts data which identifies the customer
 * and group that owns the asset.
 *
 * \param computer	The ID of the computer
 * \param customer  The node-id of the customer
 * \param group     The node-id of the group
 */
function erp_asset_insert_ownership($computer, $customer, $group)
{
  $modelid = erp_asset_find_ownership_id($customer, $group);
  if(!$modelid) {
    return NULL;
  }

	$sql = <<<CUSTSQL

		INSERT INTO
			{erp_asset_ownership}
			(computerid, modelid)
		VALUES
			(%d, %d)

CUSTSQL;
	db_query($sql, $computer, $modelid);
  return TRUE;
}

/**
 * Inserts data holding information about the installed OS.
 *
 * \param computer			The ID of the computer which we are inserting data for
 * \param os						The name of the OS
 * \param installdate		The date which the OS was installed on
 */
function erp_asset_insert_os($computer, $os, $installdate, $productid, $productcode)
{
  $debug = variable_get('asset_debug', FALSE);
	$sql = <<<DELSQL

		DELETE FROM
			{erp_asset_os}
		WHERE
			computerid = %d

DELSQL;

	db_query($sql, $computer);

	$id = erp_asset_find_os_id($os);
	$sql = <<<OSSQL

		INSERT INTO
			{erp_asset_os}
			(computerid, modelid, installdate, productid, productcode)
		VALUES
			(%d, %d, '%s', '%s', '%s')

OSSQL;
	db_query($sql, $computer, $id, $installdate, $productid, $productcode);
}

/**
 * Inserts data holding information about the installed software.
 *
 * \param computer			The ID of the computer which we are inserting data for
 * \param name					The name of the OS
 * \param version				The version of the software
 */
function erp_asset_insert_sw($computer, $name, $version, $productid, $productcode)
{
  $debug = variable_get('asset_debug', FALSE);
	$id = erp_asset_find_sw_id($name, $version);
	$sql = <<<INSSQL

		INSERT INTO
			{erp_asset_software}
			(computerid, modelid, productid, productcode)
		VALUES
			(%d, %d, '%s', '%s')

INSSQL;
	db_query($sql, $computer, $id, $productid, $productcode);
}

/**
 * Inserts data holding information about an NIC.
 *
 * \param computer			The ID of the computer which we are inserting data for
 * \param name					The name of the NIC
 * \param mac						The MAC address of the NIC
 * \param ipaddress			The first IP address of the NIC
 */
function erp_asset_insert_nic($computer, $name, $mac, $ipaddress)
{
	$debug = variable_get('asset_debug', FALSE);

	$sql = <<<DELSQL

		DELETE FROM
			{erp_asset_nic}
		WHERE
			mac = '%s'

DELSQL;

	db_query($sql, $mac);

	$id = erp_asset_find_nic_id($name);
	$sql = <<<NICSQL

		INSERT INTO
			{erp_asset_nic}
			(computerid, modelid, mac, ipaddress)
		VALUES
			(%d, %d, '%s', '%s')

NICSQL;
	db_query($sql, $computer, $id, $mac, $ipaddress);
}

/**
 * Sets the floppy flag to true.
 *
 * This function sets the "floppy-present" flag in the database to true.
 * This does not however allow properties of the drive to be set, or the
 * input of multiple floppy drives.  This limitation is not a signigicant
 * one however, since most computers contain either a single 1.44MB
 * High-Density drive, or none at all.
 *
 * \param computer	The computer containing the floppy drive
 * \param status		The status of the computer's floppy drive
 */
function erp_asset_insert_floppy($computer, $status)
{
	$flag = ($status) ? 1 : 0;
	$sql = <<<INSERTSQL

		INSERT INTO
			{erp_asset_floppy}
			(computerid, modelid)
		VALUES
			(%d, %d)
INSERTSQL;

	db_query($sql, $computer, $status);
}

/**
 * Inserts data holding information about the system as a whole.
 *
 * This function inserts data regarding the entire system into the database.
 * It takes care of the model and domain insertion too.  This is the first
 * function that you want to use when adding a computer's information, because
 * inserting all the information on the peripherals and having the system
 * automagically inserted in from the node object is error-prone and ties us to
 * Drupal.
 *
 * \param nid						The node's ID
 * \param hostname			The system's hostname
 * \param serialnumber	The serial number of the system
 * \param user					The user on the system, if applicable
 * \param reserved			'Y' if the system is reserved
 * \param domain				The domain name of the system
 * \param vendor				The system's vendor
 * \param model					The model of the system
 * \param version				The version of the system
 */
function erp_asset_insert_computer_system($nid, $hostname, $serialnumber, $user,
																				$reserved, $domain, $vendor, $model,
																				$version)
{
	$domain_id		=	erp_asset_find_domain_id($domain);
	$model_id			=	erp_asset_find_model_id($vendor, $model,
																				$version);

	$registerdate	= date('y-m-d');

	$sql = <<<INSERTSQL

			INSERT INTO
				{erp_asset_computer_system}
				(nid, hostname, model, serialnumber,
				 domain, user, reserved, registerdate)
			VALUES
				(%d, '%s', '%s', '%s', %d, '%s', '%s', '%s')

INSERTSQL;

				db_query($sql, $nid, $hostname, $model_id, $serialnumber,
									$domain_id, $user, $reserved, $registerdate);

}

/**
 * Finds a computer's node ID by querying for the serial number.
 *
 * This function uses the serial number of the computer as a unique identifier
 * to get a node for a specific computer.
 *
 * \param serialnumber	The serial number to search for
 *
 * \return 							The ID of the node containing this system, -1 if not
 *											present,  or NULL on error
 */
function erp_asset_find_system_by_serial($serialnumber)
{
	$sql = <<<FINDSQL

		SELECT
			nid
		FROM
			{erp_asset_computer_system}
		WHERE
			serialnumber = '%s'
		LIMIT 1

FINDSQL;

	$result = db_query($sql, $serialnumber);
	if(db_error($result))
	{
		return NULL;
	}

	if(db_num_rows($result) == 0)
	{
		return -1;
	}

	$nid = db_result($result);
	return $nid;
}

/**
 * Finds a computer's node ID by querying for the MAC address.
 *
 * This function uses the MACs of the computer as a unique identifier
 * to get a node for a specific computer.
 *
 * \param macs		An array of MACs to search for
 *
 * \return 				The ID of the node containing this system, -1 if not
 *									present, or NULL on error
 */
function erp_asset_find_system_by_macs($macs)
{
	$debug = variable_get('asset_debug', FALSE);

	$sql = <<<FINDSQL

		SELECT
			nid
		FROM
			{erp_asset_computer_system}
		LEFT JOIN
			{erp_asset_nic}
		ON
			{erp_asset_computer_system}.computerid
				=
			{erp_asset_nic}.computerid
		WHERE

FINDSQL;

	$first = TRUE;
	foreach($macs as $mac)
	{
		if(FALSE == $first)
		{
			$sql .= "OR\n";
		}
    else
    {
      $first = FALSE;
    }
		$sql .= "mac = '%s'\n";
	}

	$sql .= 'LIMIT 1';

	$result = db_query($sql, $macs);
	if(db_error($result))
	{
		return NULL;
	}

	$rows = db_num_rows($result);
	if($rows == 0)
	{
		return -1;
	}

	$nid = db_result($result);
	return $nid;
}

/**
 * Finds a computer's ID from its node id.
 *
 * This function translates from drupal's node numbering scheme to our computer
 * numbering scheme.
 *
 * \param nid		The nid of the computer system
 *
 * \return			The computerid of the system
 */
function erp_asset_find_computerid($nid)
{
	$sql = <<<FINDSQL

		SELECT
			computerid
		FROM
			{erp_asset_computer_system}
		WHERE
			nid = %d

FINDSQL;

	$result = db_query($sql, $nid);
	if(db_error($result))
	{
		return NULL;
	}

	if(db_num_rows($result) == 1)
	{
		return db_result($result);
	}

	return NULL;
}

/**
 * Returns information about a computer's ownership.
 *
 * This function queries the database for information on the ownership
 * of a computer.  It returns data in the form of an object
 * containing all data taken from the database (customisable by changing the
 * database schema and insertion code). At this time it contains "customerid"
 * and "groupid" properties.
 *
 * \param computerid	The ID number of the computer
 *
 * \return						An object containing information on the ownership of the
 *                      asset in question.
 */
function erp_asset_find_ownership($computerid)
{
	$sql = <<<OWNERSQL

		SELECT
			customerid
		FROM
			{erp_asset_ownership}
		JOIN
			{erp_asset_ownership_category}
			ON
				{erp_asset_ownership}.modelid
					=
				{erp_asset_ownership_category}.modelid
		WHERE
			{erp_asset_ownership}.computerid = %d

OWNERSQL;

	$result = db_query($sql, $computerid);
	if(db_error($result))
	{
		return NULL;
	}

	$ownership = db_fetch_object($result);
	return $ownership;
}


/**
 * Returns information about a computer's processors.
 *
 * This function queries the database for information on the processors
 * belonging to a computer.  It returns data in the form of an object
 * containing all data taken from the database (customisable by changing the
 * database schema and insertion code). At this time it contains "name" and
 * "speed" properties.
 *
 * \param computerid	The ID number of the computer
 *
 * \return						An array of objects containing information on the
 *										processors present
 */
function erp_asset_find_processors($computerid)
{
	$sql = <<<CPUSQL

		SELECT
			name,
			speed
		FROM
			{erp_asset_processor}
		JOIN
			{erp_asset_processor_category}
			ON
				{erp_asset_processor}.modelid
					=
				{erp_asset_processor_category}.modelid
		WHERE
			{erp_asset_processor}.computerid = %d

CPUSQL;

	$result = db_query($sql, $computerid);
	if(db_error($result))
	{
		return NULL;
	}

	$processors = array();
	while($cpu = db_fetch_object($result))
	{
		$processors[] = $cpu;
	}
	return $processors;
}

/**
 * Returns information about a computer's SCSI adaptors.
 *
 * This function queries the database for information on the processors
 * belonging to a computer.  It returns data in the form of an object
 * containing all data taken from the database (customisable by changing the
 * database schema and insertion code). At this time it contains only "name"
 *
 * \param computerid	The ID number of the computer
 *
 * \return						An array of objects containing information on the
 *										adaptors present
 */
function erp_asset_find_scsi($computerid)
{
	$sql = <<<SCSISQL

		SELECT
			name
		FROM
			{erp_asset_scsi}
		JOIN
			{erp_asset_scsi_category}
			ON
				{erp_asset_scsi}.modelid
					=
				{erp_asset_scsi_category}.modelid
		WHERE
			{erp_asset_scsi}.computerid = %d

SCSISQL;

	$result = db_query($sql, $computerid);
	if(db_error($result))
	{
		return NULL;
	}

	$processors = array();
	while($cpu = db_fetch_object($result))
	{
		$processors[] = $cpu;
	}
	return $processors;
}

/**
 * Returns information about a computer's CDROM drives.
 *
 * This function queries the database for information on the CDROM drives
 * belonging to a computer.  It returns data in the form of an object
 * containing all data taken from the database (customisable by changing the
 * database schema and insertion code). At this time it contains only "name"
 *
 * \param computerid	The ID number of the computer
 *
 * \return						An array of objects containing information on the
 *										drives present
 */
function erp_asset_find_cdrom($computerid)
{
	$sql = <<<FINDSQL

		SELECT
			name
		FROM
			{erp_asset_cdrom}
		JOIN
			{erp_asset_cdrom_category}
			ON
				{erp_asset_cdrom}.modelid
					=
				{erp_asset_cdrom_category}.modelid
		WHERE
			{erp_asset_cdrom}.computerid = %d

FINDSQL;

	$result = db_query($sql, $computerid);
	if(db_error($result))
	{
		return NULL;
	}

	$drives = array();
	while($drive = db_fetch_object($result))
	{
		$drives[] = $drive;
	}
	return $drives;
}

/**
 * Returns information about a computer's hard drives
 *
 * This function queries the database for information on the hard drives
 * belonging to a computer.  It returns data in the form of an object
 * containing all data taken from the database (customisable by changing the
 * database schema and insertion code). At this time it contains "name" and
 * "size" properties.
 *
 * \param computerid	The ID number of the computer
 *
 * \return						An array of objects containing information on the
 *										hard drives present
 */
function erp_asset_find_harddisks($computerid)
{
	$sql = <<<CPUSQL

		SELECT
			name,
			size
		FROM
			{erp_asset_harddisk}
		JOIN
			{erp_asset_harddisk_category}
			ON
				{erp_asset_harddisk}.modelid
					=
				{erp_asset_harddisk_category}.modelid
		WHERE
			{erp_asset_harddisk}.computerid = %d

CPUSQL;

	$result = db_query($sql, $computerid);
	if(db_error($result))
	{
		return NULL;
	}

	$hard_drives = array();
	while($drive = db_fetch_object($result))
	{
		$hard_drives[] = $drive;
	}
	return $hard_drives;
}

/**
 * Returns information on the RAM present in a system.
 *
 * This function extracts data from the database pertaining to a computer's
 * RAM.  At this time only the size is available, but other platforms may
 * provide more information, and as a result this function will not be tied
 * to simply returning the amount.
 *
 * \param computerid	The ID of the computer which is to be queried.
 *
 * \return						An object containing all data extracted from the database.
 *										At the time of writing, there is only a "size" attribute.
 */
function erp_asset_find_ram($computerid)
{
	$sql = <<<RAMSQL

		SELECT
			modelid		as size
		FROM
			{erp_asset_ram}
		WHERE
			computerid = %d

RAMSQL;

	$result = db_query($sql, $computerid);
	if(db_error($result))
	{
		return NULL;
	}

	if(db_num_rows($result) != 1)
	{
		return NULL;
	}

	$retval = db_fetch_object($result);

	return $retval;
}

/**
 * Returns information on the OS installed on a system.
 *
 * This function queries the database to find data on the OS installed on a
 * computer.  At this time, the following is available:
 *	- Name
 *	- Installation date
 *
 * \param computerid	The ID of the computer which is to be queried.
 *
 * \return						An object containing all data extracted from the database
 *
 * \todo							Integrate this with the "product" data to find extra
 *										information (such as CD Keys on platforms where this is
 *										relevent)
 */
function erp_asset_find_os($computerid)
{
	$sql = <<<OSSQL

		SELECT
			name,
			installdate,
      productid,
      productcode
		FROM
			{erp_asset_os}
		JOIN
			{erp_asset_os_category}
		ON
			{erp_asset_os}.modelid
				=
			{erp_asset_os_category}.modelid
		WHERE
			computerid = %d

OSSQL;

	$result = db_query($sql, $computerid);
	if(db_error($result))
	{
		return NULL;
	}

	if(db_num_rows($result) != 1)
	{
		return NULL;
	}

	return db_fetch_object($result);
}

/**
 * Returns information on the software installed on a system.
 *
 * This function queries the database to find data on the OS installed on a
 * computer.  At this time, the following is available:
 *	- Name
 *	- Version
 *
 * \param computerid	The ID of the computer which is to be queried.
 *
 * \return						An object containing all data extracted from the database
 *
 * \todo							Integrate this with the "product" data to find extra
 *										information (such as CD Keys on platforms where this is
 *										relevent)
 */
function erp_asset_find_sw($computerid)
{
	$sql = <<<SWSQL

		SELECT
			name				as name,
			sw_version	as version,
      productid   as productid,
      productcode as productcode
		FROM
			{erp_asset_software}
		JOIN
			{erp_asset_software_category}
		ON
			{erp_asset_software}.modelid
				=
			{erp_asset_software_category}.modelid
		WHERE
			computerid = %d

SWSQL;

	$result = db_query($sql, $computerid);
	if(db_error($result))
	{
		return NULL;
	}

	$retval = array();

	while($row = db_fetch_object($result))
	{
		$retval[] = $row;
	}

	return $retval;
}

/**
 * Returns information on the NICs present in a system.
 *
 * This function queries the database to find information on a computer's
 * network adaptor.  This data is returned as an object containing these
 * attributes:
 * 	- name
 * 	-	mac
 * 	- ip
 *
 * \param computerid	The ID of the computer to find data on
 *
 * \return						An array of NIC objects (described above), or NULL on
 * 										error.
 */
function erp_asset_find_nics($computerid)
{
	$debug = variable_get('asset_debug', FALSE);
	$sql = <<<FINDSQL

		SELECT
			name			as name,
			mac				as mac,
			ipaddress	as ip
		FROM
			{erp_asset_nic}
		JOIN
			{erp_asset_nic_category}
		ON
			{erp_asset_nic}.modelid
				=
			{erp_asset_nic_category}.modelid
		WHERE
			computerid = %d

FINDSQL;

	$result = db_query($sql, $computerid);
	if(db_error($result))
	{
		return NULL;
	}

	$rows = db_num_rows($result);

	$retval = array();
	while($value = db_fetch_object($result))
	{
		$retval[] = $value;
	}

	return $retval;
}

/**
 * Checks for the presence of a floppy drive.
 *
 * This function uses information in the database to check whether or not
 * the computer pointed to by the provided computer id contains a floppy
 * drive.  This function does <b>not</b> provide the number of floppy
 * drives available, or their capabilities.
 *
 * \param computerid	Computer to check
 *
 * \return						A boolean value indicating the presence of at least one
 * 										drive, or NULL on error.
 *
 * \todo							Rewrite to provide information on individual floppy
 * 										drives in a system and their capabilities.
 */
function erp_asset_find_floppy($computerid)
{
	$sql = <<<FINDSQL

		SELECT
			modelid
		FROM
			{erp_asset_floppy}
		WHERE
			computerid = %d

FINDSQL;

	$result = db_query($sql, $computerid);
	if(db_error($result))
	{
		return NULL;
	}

	if(db_num_rows($result) != 1)
	{
		return NULL;
	}

	$flag = db_result($result);
	return ($flag == 1) ? TRUE : FALSE;
}

/**
 * Returns information about a computer's VGA adaptors.
 *
 * This function queries the database for information on the VGA adaptors
 * belonging to a computer.  It returns data in the form of an object
 * containing all data taken from the database (customisable by changing the
 * database schema and insertion code). At this time it contains only "name"
 *
 * \param computerid	The ID number of the computer
 *
 * \return						An array of objects containing information on the
 *										adaptors present
 */
function erp_asset_find_vga($computerid)
{
	$sql = <<<SCSISQL

		SELECT
			name
		FROM
			{erp_asset_vga_controller}
		JOIN
			{erp_asset_vga_category}
			ON
				{erp_asset_vga_controller}.modelid
					=
				{erp_asset_vga_category}.modelid
		WHERE
			{erp_asset_vga_controller}.computerid = %d

SCSISQL;

	$result = db_query($sql, $computerid);
	if(db_error($result))
	{
		return NULL;
	}

	$adaptors = array();
	while($adaptor = db_fetch_object($result))
	{
		$adaptors[] = $adaptor;
	}
	return $adaptors;
}

/**
 * Returns information about a computer's sound adaptors.
 *
 * This function queries the database for information on the sound adaptors
 * belonging to a computer.  It returns data in the form of an object
 * containing all data taken from the database (customisable by changing the
 * database schema and insertion code). At this time it contains only "name"
 *
 * \param computerid	The ID number of the computer
 *
 * \return						An array of objects containing information on the
 *										adaptors present
 */
function erp_asset_find_sound($computerid) {
	$sql = "SELECT name FROM {erp_asset_sound_controller} ascon JOIN {erp_asset_sound_category} ascat ON ascon.modelid = ascat.modelid WHERE ascon.computerid = %d";

	$result = db_query($sql, $computerid);
	if (db_error($result)) {
		return NULL;
	}

	$adaptors = array();
	while($adaptor = db_fetch_object($result)) {
		$adaptors[] = $adaptor;
	}
	return $adaptors;
}

/**
 * Returns information about a computer.
 *
 * This function is intended to supply information on the computer suitable for
 * use in a page detailing specific information.
 *
 * \param nid	The nid to load
 * \return		An object containing the relevant data.
 */
function erp_asset_computer_load($nid) {
	$debug = variable_get('asset_debug', FALSE);
  $sql = "SELECT n.nid, hostname, serialnumber, user, computerid, amc.vendor as vendor, adc.name as domain, amc.model as model, amc.version as version, reserved " .
         "FROM node n INNER JOIN {erp_asset_computer_system} acs ON n.nid = acs.nid " .
         "INNER JOIN {erp_asset_domain_category} adc on acs.domain = adc.modelid " .
         "INNER JOIN {erp_asset_model_category} amc ON acs.model = amc.modelid " .
         "WHERE n.nid = %d";

	$result = db_query($sql, $nid);

	$retval = db_fetch_object($result);
	if ($retval->reserved != 'Y') {
		$retval->reserved = FALSE;
	}
	else {
		$retval->reserved = TRUE;
	}

	$retval->floppy = erp_asset_find_floppy($retval->computerid);

	$retval->processors = erp_asset_find_processors($retval->computerid);
	if(NULL == $retval->processors) {
		$retval->processors = array();
	}

	$retval->ram = erp_asset_find_ram($retval->computerid);
	if(NULL == $retval->ram) {
		$retval->ram = (object)array();
	}

	$retval->os = erp_asset_find_os($retval->computerid);
	if(NULL == $retval->os) {
		$retval->os = (object)array();
	}

	$retval->nic = erp_asset_find_nics($retval->computerid);
	if(NULL == $retval->nic) {
		$retval->nic = array();
	}

	$retval->harddisks = erp_asset_find_harddisks($retval->computerid);
	if(NULL == $retval->nic) {
		$retval->harddisks = array();
	}

	$retval->scsi = erp_asset_find_scsi($retval->computerid);
	if(NULL == $retval->scsi) {
		$retval->scsi = array();
	}

	$retval->cdrom = erp_asset_find_cdrom($retval->computerid);
	if(NULL == $retval->cdrom) {
		$retval->cdrom = array();
	}

	$retval->vga = erp_asset_find_vga($retval->computerid);
	if(NULL == $retval->vga) {
		$retval->vga = array();
	}

	$retval->sound = erp_asset_find_sound($retval->computerid);
	if(NULL == $retval->sound) {
		$retval->sound = array();
	}

	$retval->sw = erp_asset_find_sw($retval->computerid);
	if(NULL == $retval->sw) {
		$retval->sw = array();
	}

  $ownership = erp_asset_find_ownership($retval->computerid);

  $retval->customer_id = $ownership->customerid;
  $customer = node_load(array('nid' => $ownership->customerid,
                              'type' => 'customer'));

  $retval->customer = $customer->title;

  //$retval->group = $ownership->groupid;

	return $retval;
}

/**
 * Checks for duplicate nodes.
 *
 * This function uses the provided data to test for duplicate nodes.  This
 * allows the server to safely delete any previous information before inserting
 * new data, or update it instead.
 *
 * \param node	The node containing the data to check for
 *
 * \return			The nid of the node, -1 if no node present, or NULL on an error
 */
function erp_asset_find_duplicate_node($node) {
	$debug = variable_get('asset_debug', FALSE);
	$method = variable_get('asset_identification_method', 0);
	if (0 == $method) { //Work from client-provided information
		if (strlen(trim($node->serialnumber)) == 0) {
			$node->serialnumber = NULL;
		}
		if ($node->serialnumber) {
			return erp_asset_find_system_by_serial($node->serialnumber);
		}

		if (count($node->nic) > 0) {
			$macs = array();
			foreach ($node->nic as $nic) {
				if ($nic->mac) {
					$macs[] = $nic->mac;
				}
			}
			return erp_asset_find_system_by_macs($macs);
		}
		else {
			return NULL;
		}
	}
	else {
		if ($debug) {
			echo "Unimplemented ID method selected.\n";
		}
		return NULL;
	}
}
